# Copyright 2021 - 2026 Universität Tübingen, DKFZ, EMBL, and Universität zu Köln
# for the German Human Genome-Phenome Archive (GHGA)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Models for configuration used in the 'rename id property' transformation."""

from pydantic import Field
from pydantic_settings import BaseSettings


class RenameIdPropertyConfig(BaseSettings):
    """A model describing a configuration for renaming model's id property."""

    class_name: str = Field(
        default=...,
        description="The name of the class whose id property will be renamed.",
    )
    id_property_name: str = Field(
        default=..., description="The name of the new id property."
    )
    description: str | None = Field(
        default=None,
        description="A description of the id property name.",
    )
